package francois.pessaux.pinpoint.android ;

import java.io.DataOutputStream ;
import java.io.File ;
import java.io.FileOutputStream ;
import java.io.IOException ;

import android.app.Activity;
import android.content.Context;
import android.location.Criteria;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.os.Environment ;
import android.text.format.Time ;
import android.view.View ;
import android.view.WindowManager ;
import android.widget.CheckBox ;
import android.widget.EditText ;
import android.widget.RadioGroup ;
import android.widget.SeekBar ;
import android.widget.SeekBar.OnSeekBarChangeListener ;
import android.widget.TextView;
import android.widget.Toast;


public class MainActivity extends Activity
    implements LocationListener, OnSeekBarChangeListener {
  private static String gpxHeader =
"<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>\n" +
"<gpx xmlns=\"http://www.topografix.com/GPX/1/1\" creator=\"PinPoint\" version=\"0.2\"\n" +
"     xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n"+
"     xsi:schemaLocation=\"http://www.topografix.com/GPX/1/1 http://www.topografix.com/GPX/1/1/gpx.xsd\">\n" +
"<trk>\n" +
"  <trkseg>\n"
;
  private TextView textViewLatitute ;
  private TextView textViewLongitude ;
  private TextView textViewAccuracy ;
  private TextView textViewRecpts ;
  private CheckBox checkBoxFilter ;
  private SeekBar seekBarUpdGPS ;
  private TextView textViewUpdGPS ;
  private EditText editTextOtherEvent ;
  private RadioGroup radioGroupEvent ;
  private LocationManager locationManager ;
  private String provider ;
  private Location prevLocation = null ;
  private Location currLocation = null ;
  private String baseFileName ;
  private DataOutputStream outStream = null ;
  private boolean filterFlag = false ;
  private boolean recordingFlag = false ;
  private long nbRecpts = 0 ;
  private long updateMsGPS = 1500 ;
  private String pendingNotif = null ;
  
  
  /* ***********************************************************************
      INITIALISATION STUFF.
      *********************************************************************** */
  private boolean createOutDataFile ()
  {
    Time currDate = new Time () ;
    currDate.setToNow () ;
    /* Create the raw data out file. */
    File sdCard = Environment.getExternalStorageDirectory () ;
    baseFileName =
      PinPointApp.dataDirectoryName + File.separatorChar + "pinptrk_" +
      currDate.format2445 () + ".gpx" ;
    File outFile = new File (sdCard, baseFileName) ;
    Toast.makeText
      (this, "Data -> "+ outFile.getAbsolutePath (), Toast.LENGTH_LONG).show () ;
    /* Now, create the data file. */
    try {
      outFile.createNewFile () ;
      outStream = new DataOutputStream (new FileOutputStream (outFile)) ;
      /* Write header. */
      outStream.writeBytes (gpxHeader) ;
      return (true) ;
    }
    catch (IOException e) { return (false) ; } // TODO
  }
  
  
  private boolean acquireGPS ()
  {
    /* Get the location manager. */
    locationManager =
      (LocationManager) getSystemService (Context.LOCATION_SERVICE) ;
    // TODO: Test if NULL
    /* boolean enabled = service
        .isProviderEnabled(LocationManager.GPS_PROVIDER);

      // check if enabled and if not send user to the GSP settings
      // Better solution would be to display a dialog and suggesting to 
      // go to the settings
      if (!enabled) {
        Intent intent = new Intent(Settings.ACTION_LOCATION_SOURCE_SETTINGS);
        startActivity(intent);
      } /*
    /* Define the criteria how to select the location provider. Use default. */
    Criteria criteria = new Criteria () ;
    provider = locationManager.getBestProvider (criteria, false) ;
    currLocation = locationManager.getLastKnownLocation (provider) ;
    
    /* Initialize the location fields with "Unknown" by default. */
    textViewLatitute.setText ("Location not available") ;
    textViewLongitude.setText ("Location not available") ;
    textViewAccuracy.setText ("Accuracy not available") ;

    /* Do not start the acquisition now. Do not create the output file now. */
    return (true) ;
  }
  
  
  /* ***********************************************************************
      APPLICATION LIFECYCLE STUFF.
     *********************************************************************** */
  /** Called when the activity is first created. */
  @Override
  public void onCreate (Bundle savedInstanceState)
  {
    super.onCreate (savedInstanceState) ;
    setContentView (R.layout.main) ;
    textViewLatitute = (TextView) findViewById (R.id.textView02) ;
    textViewLongitude = (TextView) findViewById (R.id.textView04) ;
    textViewAccuracy = (TextView) findViewById (R.id.textView06) ;
    textViewRecpts = (TextView) findViewById (R.id.textView08) ;
    checkBoxFilter = (CheckBox) findViewById (R.id.checkBox1) ;
    seekBarUpdGPS = (SeekBar) findViewById (R.id.seekBar1) ;
    textViewUpdGPS = (TextView) findViewById (R.id.textView2) ;
    radioGroupEvent = (RadioGroup) findViewById (R.id.radioGroup1) ;
    editTextOtherEvent = (EditText) findViewById (R.id.editText1) ;
    nbRecpts = 0 ;
    textViewRecpts.setText ("0") ;
    seekBarUpdGPS.setOnSeekBarChangeListener (this) ;
    seekBarUpdGPS.setProgress (1) ; /* 1 s (min) + 0.5 s. */
    /* Force screen to remain on, but dim. */
    getWindow ().addFlags (WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON) ;
    acquireGPS () ;
  }

  
  /** Called when the app is about to be destroyed.
      Remove the location listener updates when Activity is paused. */
  @Override
  protected void onDestroy ()
  {
    super.onStop () ;
    locationManager.removeUpdates (this) ;
  }

  
  /* ***********************************************************************
     LOCATION CHANGE STUFF.
     *********************************************************************** */
  private boolean isLocationBetter (Location newLoc, Location oldLoc)
  {
    if (oldLoc == null) return (true) ;
    /* Check whether the new location fix is newer or older.*/
    long timeDelta = newLoc.getTime () - oldLoc.getTime() ;
    boolean isSignificantlyNewer = timeDelta > updateMsGPS ;
    boolean isSignificantlyOlder = timeDelta < -updateMsGPS ;

    /* If it's been more than two minutes since the current location, use the
       new location because the user has likely moved. */
    if (isSignificantlyNewer) return (true) ;
    /* If the new location is more than two minutes older, it must be worst. */
    else if (isSignificantlyOlder) return (false) ;

    // Check whether the new location fix is more or less accurate
    int accuracyDelta = (int) (newLoc.getAccuracy () - oldLoc.getAccuracy ()) ;
    boolean isLessAccurate = accuracyDelta > 0 ;
    boolean isMoreAccurate = accuracyDelta < 0 ;
    boolean isSignificantlyLessAccurate = accuracyDelta > 20 ;

    /* Determine location quality using a combination of timeliness and
       accuracy. */
    boolean isNewer = timeDelta > 0 ;
    if (isMoreAccurate) return (true) ;
    else if (isNewer && !isLessAccurate) return (true) ;
    else if (isNewer && !isSignificantlyLessAccurate) return (true) ;
    return (false) ;
  }
  
  
  @Override
  /* Handler called each time the location changed. */
  public void onLocationChanged (Location location)
  {
    /* Only work if the new location looks better than the old one. */
    if (!filterFlag || (isLocationBetter (location, prevLocation))) {
      double lat = location.getLatitude () ;
      double lng = location.getLongitude () ;
      float acc = location.getAccuracy () ;
      /* Set text. */
      textViewLatitute.setText (String.valueOf (lat)) ;
      textViewLongitude.setText (String.valueOf (lng)) ;
      textViewAccuracy.setText (String.valueOf (acc)) ;
      /* Update the number of recorded points. */
      nbRecpts++ ;
      textViewRecpts.setText (String.valueOf (nbRecpts)) ;
      /* Record location in the register. */
      try {
        outStream.writeBytes ("    <trkpt lat=\"") ;
        outStream.writeBytes (String.valueOf (lat)) ;
        outStream.writeBytes ("\" lon=\"") ;
        outStream.writeBytes (String.valueOf (lng)) ;
        outStream.writeBytes ("\">\n      <cmt>\"") ;
        outStream.writeBytes (String.valueOf (acc)) ;
        outStream.writeBytes ("\"</cmt>\n") ;
        /* If there is a pending notification, write it as the point's name. */
        if (pendingNotif != null) {
          outStream.writeBytes ("      <name>\"") ;
          outStream.writeBytes (pendingNotif) ;
          outStream.writeBytes ("\"</name>\n") ;
          /* Clear the pending notification. */
          pendingNotif = null ;
        }
        outStream.writeBytes ("    </trkpt>\n") ;
      }
      catch (IOException e) { } // TODO
      /* Update previous (old) location. */
      prevLocation = location ;
      }
    }

  
  /* ***********************************************************************
     LOCATION PROVIDER STUFF.
     *********************************************************************** */
  @Override
  public void onStatusChanged (String provider, int status, Bundle extras)
  {
    // TODO Auto-generated method stub

  }

  @Override
  public void onProviderEnabled (String provider)
  {
    Toast.makeText (this, "Location provider on: " + provider,
        Toast.LENGTH_SHORT).show () ;
  }

  @Override
  public void onProviderDisabled (String provider) {
    Toast.makeText(this, "Location provider off: " + provider,
        Toast.LENGTH_SHORT).show () ;
  }
  
  
  /* ***********************************************************************
     UI STUFF STUFF.
     *********************************************************************** */
  public void onClickFilterBox (View view)
  {
    filterFlag = checkBoxFilter.isChecked () ;
  }
  
  /** Handler called then the button "Start" is pressed. Launch listening of
      location changes. */
  public void onClickStart (View view)
  {
    createOutDataFile () ;
    recordingFlag = true ;
    /* Request for at least 1 meter distance. */
    locationManager.requestLocationUpdates (provider, updateMsGPS, 1, this) ;
    Toast.makeText (this, "Let's go!", Toast.LENGTH_SHORT).show () ;
  }
  
  
  /** Handler called then the button "Record" is pressed. */
  public void onClickNotify (View view)
  {
    int selectedId = radioGroupEvent.getCheckedRadioButtonId () ;
    /* For debug.
    RadioButton radioButt = (RadioButton) findViewById (selectedId) ;
    Toast.makeText (this, radioButt.getText (), Toast.LENGTH_SHORT).show () ;
    */
    switch (selectedId) {
    case R.id.radio0: pendingNotif = "30" ; break ;
    case R.id.radio1: pendingNotif = "50" ; break ;
    case R.id.radio2: pendingNotif = "70" ; break ;
    case R.id.radio3: pendingNotif = "90" ; break ;
    case R.id.radio5: pendingNotif = "110" ; break ;
    case R.id.radio6:
      pendingNotif = editTextOtherEvent.getText ().toString () ;
      break ;
    default: break ;
    }
  }
  
  /** Handler called then the button "Quit" is pressed. */
  public void onClickQuit (View view)
  {
    try {
      if (outStream != null) {
        /* Write trailer. */
        outStream.writeBytes ("</trkseg></trk></gpx>\n") ;
        /* Close the out data file. */
        outStream.close () ;
      }
    }
    catch (IOException e) { } // TODO
    /* Quit and let system housework be done. */
    Toast.makeText (this, "Bye!", Toast.LENGTH_SHORT).show () ;
    finish () ;
  }


  @Override
  public void onProgressChanged (SeekBar seekBar, int progress, boolean fromUser)
  {
    double newSecDelay ;
    /* Simply update the displayed text. Do not really change the GPS listener
       delay. */
    newSecDelay = 1.0 + ((double) progress) * 0.5 ;
    textViewUpdGPS.setText ((String.valueOf (newSecDelay)) + "s") ;
  }


  @Override
  public void onStartTrackingTouch (SeekBar seekBar) { /* Do nothing. */ }


  @Override
  public void onStopTrackingTouch (SeekBar seekBar)
  {
    /* In fact, 1000 + (1000 * progress * 0.5). */
    long newUpdMsGPS = 100 + (500 * seekBar.getProgress ()) ;
    if (newUpdMsGPS != updateMsGPS) {
      /* New update delay. Update it. */
      updateMsGPS = newUpdMsGPS ;
      /* Now, if we are recording, then stop the current listener and start
         another with the new delay. */
      if (recordingFlag) {
        locationManager.removeUpdates (this) ;
        locationManager.requestLocationUpdates (provider, updateMsGPS, 1, this) ;
        Toast.makeText (this, "Delay changed.", Toast.LENGTH_SHORT).show () ;
      }
    }
  }

}
